"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.useAsync = useAsync;
exports.useFetch = useFetch;

var _vueFunctionApi = require("vue-function-api");

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/**
 * Async helper function that returns three reactive values:
 * * `isLoading`, a boolean that is true during pending state;
 * * `data`, contains the resolved value in the fulfilled state; and
 * * `error`, contains the exception in the rejected state.
 *
 * @param {function} promiseFn A function that returns a Promise.
 * @param {object} params Parameters passed as first argument to the promise function.
 * @returns {object} Object literal containing `isLoading`, `error` and `data` value wrappers.
 */
function useAsync(promiseFn, params) {
  var _this = this;

  var isLoading = (0, _vueFunctionApi.value)(true);
  var error = (0, _vueFunctionApi.value)(null);
  var data = (0, _vueFunctionApi.value)(null);
  var controller = null;

  (0, _vueFunctionApi.onCreated)(_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var _controller, signal, result;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;

            controller = new AbortController();
            _controller = controller, signal = _controller.signal;
            _context.next = 5;
            return promiseFn(params, signal);

          case 5:
            result = _context.sent;

            data.value = result;
            _context.next = 12;
            break;

          case 9:
            _context.prev = 9;
            _context.t0 = _context["catch"](0);

            error.value = _context.t0;

          case 12:
            _context.prev = 12;

            isLoading.value = false;
            return _context.finish(12);

          case 15:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, _this, [[0, 9, 12, 15]]);
  })));

  (0, _vueFunctionApi.onBeforeDestroy)(function () {
    if (controller !== null) {
      controller.abort();
    }
  });

  return {
    isLoading: isLoading,
    error: error,
    data: data
  };
}

/**
 * Fetch helper function that accepts the same arguments as `fetch` and returns the same reactive values as `useAsync`.
 * If the `Accept` header is set to `application/json` in the `requestInit` object, the response will be parsed as JSON,
 * else text.
 *
 * @param {string|object} requestInfo URL or request object.
 * @param {object} requestInit Init parameters for the request.
 * @returns {object} Object literal containing `isLoading`, `error` and `data` value wrappers.
 */
function useFetch(requestInfo) {
  var doFetch = function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(params, signal) {
      var res;
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return fetch(requestInfo, _extends({}, requestInit, {
                signal: signal
              }));

            case 2:
              res = _context2.sent;

              if (res.ok) {
                _context2.next = 5;
                break;
              }

              throw new Error(res);

            case 5:
              if (!(requestInit.headers && requestInit.headers.Accept === "application/json")) {
                _context2.next = 7;
                break;
              }

              return _context2.abrupt("return", res.json());

            case 7:
              return _context2.abrupt("return", res.text());

            case 8:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    return function doFetch(_x2, _x3) {
      return _ref2.apply(this, arguments);
    };
  }();

  var requestInit = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};


  return useAsync(doFetch);
}
//# sourceMappingURL=index.js.map